package net.mc_pandacraft.java.plugin.pandacraftutils.data_model;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import net.mc_pandacraft.java.plugin.pandacraftutils.commands.AbstractCommandExecutor;

public class MPMessageTable extends SQLTable<MPMessageElement> {

	public MPMessageTable() throws SQLException {
		super("pandacraft_mp_message");
	}

	@Override
	protected String createTableParameters() {
		return "id INT AUTO_INCREMENT PRIMARY KEY,"
				+ "time BIGINT NOT NULL,"
				+ "securityKey INT NOT NULL,"
				+ "viewerNick VARCHAR(16) NOT NULL,"
				+ "sourceNick VARCHAR(16) NULL," // Null si la source est la console ou une autre entité qu'un joueur
				+ "destNick VARCHAR(16) NULL,"
				+ "destGroup INT NULL,"
				+ "message VARCHAR(512) NOT NULL,"
				+ "wasRead TINYINT NOT NULL,"
				+ "deleted TINYINT NOT NULL,"
				+ "serverSync TINYINT NOT NULL";
	}

	@Override
	protected MPMessageElement getElementInstance(ResultSet sqlResult)
			throws SQLException {
		MPMessageElement el = new MPMessageElement(
				sqlResult.getInt("id"),
				sqlResult.getLong("time"),
				sqlResult.getInt("securityKey"),
				sqlResult.getString("viewerNick"),
				sqlResult.getString("sourceNick"),
				sqlResult.getString("message"),
				sqlResult.getBoolean("wasRead"),
				sqlResult.getBoolean("serverSync"));
		el.setDestNick(sqlResult.getString("destNick"));
		
		int group = sqlResult.getInt("destGroup");
		el.setDestGroup(sqlResult.wasNull()?null:group);
		
		el.setDeleted(sqlResult.getBoolean("deleted"));
		
		return el;
	}
	
	
	
	
	
	public List<MPMessageElement> getAllUnsyncMessage() throws SQLException {
		return getAll("serverSync = 0", "time ASC", null, null);
	}
	
	
	
	public List<MPMessageElement> getAllUnreadForPlayer(String player) throws SQLException {
		if (!AbstractCommandExecutor.isValidPlayerName(player)) return null;
		
		return getAll("viewerNick = '"+player+"' AND wasRead = 0", "time ASC", null, null);
	}
	

}
