package net.mc_pandacraft.java.plugin.pandacraftutils.data_model;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

/**
 * <b>ORM = Object-Relational Mapping</b><br/>
 * Liste des tables avec leur classes :
 * <ul>
 * 	<li><code>"modo_history" : ModoHistoryTable</code></li>
 * 	<li><code>"staff_ticket" : StaffTicketTable</code></li>
 * 	<li><code>"mp_message" : MPMessageTable</code></li>
 * 	<li><code>"mp_group" : MPGroupTable</code></li>
 * 	<li><code>"mp_group_user" : MPGroupUserTable</code></li>
 * 	<li><code>"mp_web_session" : MPWebSessionTable</code></li>
 * </ul>
 * @author Marc Baloup
 *
 */
public final class ORM {
	
	@SuppressWarnings("rawtypes")
	private static Map<String, SQLTable> tables = new HashMap<String, SQLTable>();

	
	public synchronized static void init() {
		try {
			
			/*
			 * Les tables SQL sont à instancier ici !
			 */

			tables.put("modo_history", new ModoHistoryTable());
			
			tables.put("staff_ticket", new StaffTicketTable());

			tables.put("mp_message", new MPMessageTable());
			tables.put("mp_group", new MPGroupTable());
			tables.put("mp_group_user", new MPGroupUserTable());
			tables.put("mp_web_session", new MPWebSessionTable());
			
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
	
	
	
	
	
	/**
	 * Récupère l'instance de la classe associé à la table demandé en paramètre.<br/>
	 * <b>Il est conseillé de caster le retour de cette méthode par la classe associé à la table demandé.</b>
	 * @param name nom de la table telle qu'elle est enregistrée dans la classe ORM
	 * @return ta table dont le nom est passé en paramètre
	 */
	@SuppressWarnings("rawtypes")
	public synchronized static SQLTable getTable(String name) {
		return tables.get(name);
	}
	
	
	
	
	
	

	
	private ORM() { } // rend la classe non instanciable
	
	
}
