package net.mc_pandacraft.java.plugin.pandacraftutils.players;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.mc_pandacraft.java.plugin.pandacraftutils.PandacraftUtils;

import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerQuitEvent;

public final class OnlinePlayerManager {
	
	private static OnlinePlayerManager instance;
	
	/**
	 * Retourne l'unique instance de la classe. Si elle n'existe pas, on tente de créer
	 * @return L'unique instance de la classe
	 */
	
	public synchronized static OnlinePlayerManager getInstance() {
		if (instance == null)
			loadNewInstance();
		return instance;
	}
	
	public synchronized static void loadNewInstance() {
		instance = new OnlinePlayerManager();
	}
	
	
	
	
	private Map<Player, OnlinePlayer> players = Collections.synchronizedMap(new HashMap<Player, OnlinePlayer>());
	
	private OnlinePlayerManager() {
		
		// en cas de /reload : on ajoute les joueurs déjà en ligne
		for (Player p : PandacraftUtils.getInstance().getServer().getOnlinePlayers())
			players.put(p, new OnlinePlayer(p));
		
		
        
	}
	
	
	
	
	
	// ----------------------------------------
	// -------------- Evènements --------------
	// ----------------------------------------
	public synchronized void onPlayerJoin (PlayerJoinEvent event)
	{
		players.put(event.getPlayer(), new OnlinePlayer(event.getPlayer()));
	}

	
	public synchronized void onPlayerQuit (PlayerQuitEvent event)
	{
		players.remove(event.getPlayer());
	}
	
	
	
	
	
	
	
	
	
	

	public synchronized static OnlinePlayer get(Player p) {
		return getInstance().players.get(p);
	}
	
	/**
	 * Insensible à la casse
	 * @param p le pseudo du joueur recherché
	 * @return
	 */
	public static OnlinePlayer get(String p) {
		return get(Bukkit.getServer().getPlayerExact(p));
	}
	
	public synchronized static List<OnlinePlayer> getAll() {
		return new ArrayList<OnlinePlayer>(getInstance().players.values());
	}
	
	
	public synchronized static boolean isOnline(Player p) {
		return getInstance().players.containsKey(p);
	}
	
	public static List<OnlinePlayer> getAllNotVanished() {
		List<OnlinePlayer> players = getAll();
		
		for (OnlinePlayer op : players.toArray(new OnlinePlayer[players.size()])) {
			if (op.isVanished())
				players.remove(op);
		}
		
		return players;
	}
	
	
	public static List<String> getNamesOnlyVisible(Player p) {
		List<OnlinePlayer> players = getAll();
		List<String> playerList = new ArrayList<String>();
		
		for (OnlinePlayer op : players)
			if (p == null || p.canSee(op.getPlayer()))
				playerList.add(op.getName());
		
		return playerList;
	}
	
	
	
	
	
	
	
	public static class OnlinePlayerOrderer implements Comparator<OnlinePlayer> {

		@Override
		public int compare(OnlinePlayer arg0, OnlinePlayer arg1) {
			return arg0.getName().compareToIgnoreCase(arg1.getName());
		}
		
	}
	
}
